#!/usr/bin/env perl
#######################################################
#
# Dmiry <Legh> Shulgachik
# http://legh.ru/jx     legh@legh.ru
#
# The software is provided "as is", without warranty of any kind,
# express or implied, including but not limited to the warranties
# of merchantability, fitness for a particular purpose, and
# noninfringement. In no event shall the author(s) be liable for any
# claim, damages, or other liability, whether in an action of
# contract, tort, or otherwise, arising from, out of, or in connection
# with the software or the use or other dealings in the software.
#
# SUPPORT
# Send bug reports to: <legh@legh.ru>
#
# LICENSE
# This software is in the public domain.
#
# v1.00      15.12.2008
# v1.01      31.12.2008
# v1.02      03.01.2009
# v1.10      04.01.2009
#
use constant script_version => 'v1.10';
#
#  useage
#      jx  [-j] [-l] [-h] jid|jhostname|jname [command]
#
#######################################################

use strict;
use warnings;

use constant use_config => 0;

my %config =(
	command          => 'bash',
	use_jid          => 0,
	show_tips        => 1,
);

my $jid=undef;
my %list;
my @nlist;
my @cmd;

if ( use_config )
{
	my @configpath = qw( /usr/local/etc/ /etc/ . );
	eval {
		require Config::General;
		%config = Config::General::ParseConfig(-DefaultConfig => \%config,-ConfigFile => 'jx.cfg',
							-AutoTrue=>'true',
							-MergeDuplicateOptions=> 'yes',
							-ConfigPath => \@configpath );
	} or exit_use_config();
}

sub basename {
	my @path=split('/',$0);
	my @rpath;
	foreach (@path){
		if ( $_ ne '' ) { push(@rpath,$_); }
	}
	return $rpath[$#rpath];
}

sub exit_use_config {
	print basename().": Can't load configuration file :(\n";
	exit(2);
}

sub exit_not_exec($) {
	print basename().": Can't jexec - no such jail ".(shift @_)." :(\n";
	exit(1);
}

sub version{
	print basename().' is a wrapper to "jexec", '.script_version."\n";
}

sub help {
	version();
	print "\tUsage: ".basename()."  [-j] [-h] [-l] jid|jhostname|jname [command]\n";
}

sub fill_list {
	my @rc;
	open FF, '</etc/rc.conf';
	@rc=<FF>;
	close FF;

	foreach (@rc) {
		if ( $_ =~ m/[\ \t]*jail\_([a-zA-Z0-9\ \-\_\.]*)\_hostname[\ \t]*=[\ \t]*[\'\"]?([a-zA-Z0-9\ \.\-\_]*)[\'\"]?[\ \t]?/g )
		{
			$list{$1}=$2;
			push(@nlist,$2);
		}
	}
}

sub showlist {
	if ( $config{use_jid} ){
		exec '/usr/sbin/jls';
		exit;
	}
	fill_list();
	print "\tJID\tHostname\n";
	for (my $i=0;$i<$#nlist+1;$i++){
		print "\t$i\t$nlist[$i]\n";
	}
	exit;
}

if ( $#ARGV < 0 ){
	help();
	exit(4);
}

#read options
my $cmdbegin=1;
my $showlist=0;
while ( defined (my $option=shift @ARGV) )
{
	if ( $cmdbegin ) {
		if ( $option =~ m/^(-j|--jid)$/ )
		{
			$config{use_jid}=$config{use_jid}? 0 : 1 ;
		}
		elsif ( $option =~ m/^(-l|--list|l)$/ )
		{
			$showlist=1;
		}
		elsif ( $option =~ m/^(-h|--help)$/ )
		{
			help();
			exit;
		}
		elsif ( $option =~ m/^(-v|--version)$/ )
		{
			version();
			exit;
		}
		else
		{
			showlist() if $showlist;
			$cmdbegin=0;
			push(@cmd,$option);
		}
	} else {
		push(@cmd,$option);
	}
}
showlist() if $showlist;

# if only -j options
if ( $#cmd<0 ) {
	print basename().": Can't jexec - no jail is given :(\n";
	exit(4);
}

# if no command
if ( $#cmd==0 ){
	push(@cmd,$config{command});
}

if ( $cmd[0]=~ m/^(\d+)$/ ) {
	shift @cmd;
	$jid=$1;
	if ( $config{use_jid} ) {
		my @jls=`/usr/sbin/jls`;
		shift @jls;
		my %jls;

		foreach (@jls) {
			$_ =~ m/\b(\d+)\s+\S+\s+\b([\w\-\.]*)\s+/g;
			if (length($1)!=0 ) { $jls{$1}=$2; }
		}
		if ( defined($jls{$jid}) ) {
			print "=> $jls{$jid}\n" if $config{show_tips};
			exec "/usr/sbin/jexec $jid ".join(' ', map{"'$_'"} @cmd );
			exit;
		}else{
			exit_not_exec($jid);
		}
	}
}

my @jls=`/usr/sbin/jls`;
shift @jls;
my %jls;

foreach (@jls) {
	$_ =~ m/\b(\d+)\s+\S+\s+\b([\w\-\.]*)\s+/g;
	if (length($1)!=0 ) { $jls{$2}=$1; }
}

if ( !$config{use_jid} ) {
	fill_list();
	if ( defined($jid) and defined($nlist[$jid]) ) {
		print "=> $nlist[$jid]" if $config{show_tips};
		$jid=$jls{$nlist[$jid]};
		print " $jid\n" if $config{show_tips};
		exec "/usr/sbin/jexec $jid ".join(' ', map{"'$_'"} @cmd );
		exit;
	}
}

if ( $config{use_jid} ) { fill_list(); }

if ( defined($jls{$cmd[0]}) ) {
	$jid=$jls{shift @cmd};
	print "=> $jid\n" if $config{show_tips};
	exec "/usr/sbin/jexec $jid ".join(' ', map{"'$_'"} @cmd );
	exit;
}

if ( defined($list{$cmd[0]}) and defined($jls{$list{$cmd[0]}}) ) {
	$jid=$jls{$list{$cmd[0]}};
	print "=> $list{$cmd[0]} $jid\n" if $config{show_tips};
	exec "/usr/sbin/jexec $jid ".join(' ', map{"'$_'"} @cmd );
	exit;
}

# only 1 parametr, and it is not a jid or etc.
exit_not_exec("'$cmd[0]'");
